#pragma once
#ifndef TRIM_MATH_H
#define TRIM_MATH_H
#include <random>
#include <functional>
#include <ranges>
#include <array>
/*
double get_rand()
函数用途：随机输出一个范围在0到1的double型浮点数
输入：无
输出；一个范围在0到1、均匀分布的double型浮点数
注意：此函数摘自cppreference官方示例，为提高速度对种子等变量使用了static
*/
double get_rand();
/*
double get_zero_newton(std::function<double(double)>f, std::function<double(double)>df, double x, double error)
函数用途：使用牛顿迭代法求方程的零点
x_{n+1}=x_n-f(x_n)-f'(x_n)
输入：
    f：所求零点函数
    df：所求零点函数的导数
    x：给定初始点
    xerror：给定自变量误差值
    yerror：给定因变量误差值
输出；所求零点（在给定误差范围内）
注意：此函数需要使用C++20标准（可能不用，但我没测试，且至少需要C++11标准），df需要自己算出来给定形式
*/
inline double get_zero_newton(std::function<double(double)>f, std::function<double(double)>df, double x, double xerror = 1e-14, double yerror = 1e-14) {
    while (abs(f(x)) > yerror || abs(f(x) / df(x)) >= xerror) { //牛顿迭代法
        x = x - f(x) / df(x);
    }

    return x;
}
/*
三维旋转方向矢量
输入：
direction:方向矢量
theta:偏转角
phi:方位角
*/
inline void rotate(std::array<double, 3>& direction, double theta, double phi) {
    double& l = direction[0];
    double& m = direction[1];
    double& n = direction[2];
    double x, y, z;
    if (n == 1) {
        x = sin(theta) * sin(phi);
        y = sin(theta) * cos(phi);
        z = cos(theta);
    }
    else if (n == -1) {
        x = sin(theta) * sin(phi);
        y = sin(theta) * cos(phi);
        z = -cos(theta);
    }
    else {
        x = l * cos(theta) + sin(theta) / sqrt(1 - pow(n, 2)) * (l * n * cos(phi) + m * sin(phi));
        y = m * cos(theta) + sin(theta) / sqrt(1 - pow(n, 2)) * (m * n * cos(phi) - l * sin(phi));
        z = n * cos(theta) - sqrt(1 - pow(n, 2)) * sin(theta) * cos(phi);
    }

    double len = sqrt(x * x + y * y + z * z);
    l = x / len;
    m = y / len;
    n = z / len;
}
/*
range like python
*/
inline auto Range(double start, double end, double step)
{
    if (start > end && step > 0) {
        step *= -1;
    }

    int count;
    if (step == 0) {
        count = 0;
    }
    else {
        count = (end - start) / step + 1;
    }
    auto step_fun = [=](auto x) { return x * step + start; };
    return std::views::iota(0, count)
        | std::views::transform(step_fun);
}
/*
range形式生成std::vector<double>
*/
inline std::vector<double> range_vec(double start, double end, double step) {
    std::vector<double> list;
    for (auto i : Range(start, end, step)) {
        list.push_back(i);
    }
    return list;
}
#endif // TRIM_MATH_H

